local _, class = UnitClass("player")
if class ~= "DEATHKNIGHT" then
	return
end

local plugin = mRunes:NewModule("Runes", "AceEvent-3.0")
plugin.mRunesPlugin = true
MRUNES_RUNES = plugin

local AceGUI = LibStub("AceGUI-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local UIF
local db

-- Rune Types
local RUNETYPE_BLOOD = 1;
local RUNETYPE_UNHOLY = 2;
local RUNETYPE_FROST = 3;
local RUNETYPE_DEATH = 4;

local NrRunes = 6
local InCombat = false
local RuneOnCD = false

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			HideBlizz = true,
			x = 0,
			y = 0,
			InternalSpacing = 1,
			FrameColor = {0, 0, 0, 0.5},
			
			RuneSpacing = 1,
			RuneWidth = 55,
			RuneHeight = 20,
			RuneTexture = "HalD",
			RuneAlphaInCombat = 1,
			RuneAlphaOutCombat = 0.2,
			RuneAlphaActive = 0.5,
			RuneCooldownInset = 2,
			RuneCooldownAlwaysShow = false,
			RuneCooldownShow = true, 
			RuneCooldownHorAlign = "LEFT",
			RuneCooldownVerAlign = "BOTTOM",
			
			Font = "accid",
			FontSize = 16,
			FontFlags = "OUTLINE",
			FontColor = {1, 1, 1, 1},
		
			RuneColors = {
				[RUNETYPE_BLOOD] = {1.0, 0.0, 0.0, 1},
				[RUNETYPE_UNHOLY] ={0.0, 0.8, 0.0, 1},
				[RUNETYPE_FROST] = {0.0, 1.0, 1.0, 1},
				[RUNETYPE_DEATH] = {0.8, 0.1, 1.0, 1},
			}
		}
	}
	
	self.db = mRunes.db:RegisterNamespace(self:GetName(), defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	db = self.db.profile
	
	self:SetEnabledState(db.Enabled)
	
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mRunes")
end

function plugin:OnEnable()	
	if not mRuneFrame then
		self:CreateFuneFrame()
	end

	self:ApplySettings()
	self.Frame:Show()
	
	self:RegisterEvent("RUNE_TYPE_UPDATE", "RuneTypeUpdate")
	self:RegisterEvent("RUNE_POWER_UPDATE", "RuneStateChange")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "EnterWorld") 
	
	mRunes.RegisterCallback(self, "EnteringCombat")
	mRunes.RegisterCallback(self, "LeavingCombat")
	
	if UnitAffectingCombat("player") then
		self:EnteringCombat("EnteringCombat")
	else
		self:LeavingCombat("LeavingCombat")
	end
	
end

function plugin:OnDisable()
	self.Frame:Hide()
end

function plugin:ProfileChanged()
	db = self.db.profile
	self:ApplySettings()
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Callbacks ~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:EnteringCombat(event)
	mRunes.FadeIn(self.Frame, false, db.RuneAlphaInCombat)
	InCombat = true
	RuneOnCD = false
end

function plugin:LeavingCombat(event)
	mRunes.FadeOut(self.Frame, db.AlphaOutCombat)
	InCombat = false
end

function plugin:EnterWorld(...)
	if self.Runes then
		for i, rune in pairs(self.Runes) do
			rune.Update = true
			self:RuneTypeUpdate("RUNE_TYPE_UPDATE", i)
		end
	end
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Events ~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~

-- Rune type changed (unholy runes)
function plugin:RuneTypeUpdate(event, runeID)
	self.Runes[runeID]:SetStatusBarColor(unpack(db.RuneColors[GetRuneType(runeID)]))
end

-- Rune Ready/Not Ready event
function plugin:RuneStateChange(event, runeID, ready)
	if runeID == 8 or runeID == 7 then return end
	
	if ready then
		self.Runes[runeID]:SetValue(1)		
		self.Runes[runeID].Update = nil
		self.Runes[runeID].CDText:SetText(("%.1f"):format(0))
		if not db.RuneCooldownAlwaysShow then
			self.Runes[runeID].CDText:Hide()
		end
	else
		self.Runes[runeID].Update = not ready	
	end	
end

local function Update(self, elapsed)
	self = plugin
	local haveCD = false
	local oldHaveCD = RuneOnCD
	for i, rune in pairs(self.Runes) do
		if rune.Update then
			local StartTime, CooldownTime, Ready = GetRuneCooldown(i)
			local diff = GetTime() - StartTime
			local remaining = CooldownTime - diff
			local t = diff / CooldownTime					
			rune:SetValue(t)
			if remaining < 0 and not db.RuneCooldownAlwaysShow then 
				rune.CDText:Hide() 
			elseif db.RuneCooldownShow then
				rune.CDText:Show()	
				rune.CDText:SetText(("%.1f"):format(Ready and 0 or (remaining < 0 and 0 or remaining))) 
			end
			
			if Ready then rune.Update = nil else haveCD = true end
		end
	end

	if not InCombat then
		RuneOnCD = haveCD
		if RuneOnCD and (RuneOnCD ~= oldHaveCD) then
			local currAlpha = self.Frame:GetAlpha()
			if currAlpha < db.RuneAlphaActive then
				mRunes.FadeIn(self.Frame, false, db.RuneAlphaActive)
			else
				mRunes.FadeOut(self.Frame, db.RuneAlphaActive)
			end
		elseif not RuneOnCD and (RuneOnCD ~= oldHaveCD) then
			mRunes.FadeOut(self.Frame, db.RuneAlphaOutCombat)
		end
	end
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Settings	~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:ApplySettings()
	if self.Frame then
		self.Frame:ClearAllPoints()
		
		self.Frame:SetPoint("CENTER", mRunes.MainFrame, "CENTER", db.x, db.y)
		self.Frame:SetWidth(db.RuneWidth * NrRunes + db.RuneSpacing * 5 + db.InternalSpacing * 2)
		self.Frame:SetHeight(db.RuneHeight + db.InternalSpacing * 2)
		
		self.Frame:SetAlpha(InCombat and db.RuneAlphaInCombat or db.RuneAlphaOutCombat)
		
		for i, rune in pairs(self.Runes) do
			rune:ClearAllPoints()
			if i == 1 then
				rune:SetPoint("LEFT", db.InternalSpacing, 0)
			else
				rune:SetPoint("LEFT", self.Runes[i - 1], "RIGHT", db.RuneSpacing, 0)
			end
			rune:SetWidth(db.RuneWidth)
			rune:SetHeight(db.RuneHeight)
			rune:SetStatusBarTexture(LSM:Fetch("statusbar", db.RuneTexture))
			rune:GetStatusBarTexture():SetHorizTile(false)
			rune:SetStatusBarColor(unpack(db.RuneColors[GetRuneType(i)]))
			
			rune.CDText:ClearAllPoints()		
			rune.CDText:SetPoint("TOPLEFT", db.RuneCooldownInset, -db.RuneCooldownInset)
			rune.CDText:SetPoint("BOTTOMRIGHT", -db.RuneCooldownInset, db.RuneCooldownInset)
			rune.CDText:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontFlags)
			rune.CDText:SetTextColor(unpack(db.FontColor))
			rune.CDText:SetJustifyH(db.RuneCooldownHorAlign)
			rune.CDText:SetJustifyV(db.RuneCooldownVerAlign)
			
			if db.RuneCooldownAlwaysShow and db.RuneCooldownShow then
				rune.CDText:Show()
			else
				rune.CDText:Hide()				
			end
			rune.Update = true
		end
		
		self.Frame:SetAlpha(InCombat and db.RuneAlphaInCombat or (RuneOnCD and db.RuneAlphaActive or db.RuneAlphaOutCombat))
		if self.Frame:GetAlpha() > 0 then self.Frame:Show() else self.Frame:Hide() end
		
		self.Frame.bg:SetTexture(unpack(db.FrameColor))
	end
	
	if db.HideBlizz then RuneFrame:Hide()
	else RuneFrame:Show() end
	
	if not db.Enabled and self:IsEnabled() then self:Disable()
	elseif db.Enabled and not self:IsEnabled() then self:Enable() end
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Create Frames ~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:CreateFuneFrame()
	local frame = CreateFrame("Frame", "mRunesRuneFrame", UIParent)
	frame.bg = frame:CreateTexture()
	frame.bg:SetAllPoints()
	
	mRunes:RegisterOnUpdate(frame, Update)	
	
	self.Runes = {}
	for i = 1, NrRunes do
		self.Runes[i] = self:CreateRune(i, frame)
	end
	
	frame:Show()
	
	self.Frame = frame
end

function plugin:CreateRune(runeID, parent)
	local rune = CreateFrame("StatusBar", "mRunesRune"..runeID, parent)
	rune:SetMinMaxValues(0,1)	
	rune:SetValue(1)
	
	local runeCooldown = rune:CreateFontString(nil, nil, "GameFontNormal")
	rune.CDText = runeCooldown
	
	rune.Update = true	
	rune:Show()
	
	return rune
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Options ~~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function plugin:GetOptions()
	return plugin.GetGUI
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	local FontFlagList = {
		["NONE"] = "None",
		["OUTLINE"] = "Outline",
		["THICKOUTLINE"] = "Thick Outline",
		["MONOCHROME"] = "Monochrome",
	}
	
	local VerticalAlignList = {
		["TOP"] = "Top",
		["MIDDLE"] = "Middle",
		["BOTTOM"] = "Bottom",
	}
	
	local HorizontalAlignList = {
		["LEFT"] = "Left",
		["CENTER"] = "Center",
		["RIGHT"] = "Right",
	}

	
	function plugin:GetGUI()
		self = plugin
		
		local returnGroup = AceGUI:Create("SimpleGroup")
		returnGroup:SetFullWidth(true)
		returnGroup:SetLayout("Flow")
		
		returnGroup:AddChild(UIF:CheckBox("Enabled", db, "Enabled", Callback, nil, 0.33))
		returnGroup:AddChild(UIF:CheckBox("Hide standard rune frame", db, "HideBlizz", Callback, nil, 0.33))
		
		local group = UIF:InlineGroup1("Position and Size")
		returnGroup:AddChild(group)		
		group:AddChild(UIF:Text1("Positions here are relative to the mRunes main position"))
		group:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.5))
		group:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.5))
		group:AddChild(UIF:Slider("Width", db, "RuneWidth", 5, 200, 0.1, Callback, 0.5))
		group:AddChild(UIF:Slider("Height", db, "RuneHeight", 5, 200, 0.1, Callback, 0.5))
		group:AddChild(UIF:Slider("Background Space", db, "RuneSpacing", 0, 100, 0.1, Callback, 0.5))
		
		group = UIF:InlineGroup1("Colors")
		returnGroup:AddChild(group)
		group:AddChild(UIF:ColorSelect("Blood", db.RuneColors, 1, Callback, true, 0.25))
		group:AddChild(UIF:ColorSelect("Unholy", db.RuneColors, 2, Callback, true, 0.25))
		group:AddChild(UIF:ColorSelect("Frost", db.RuneColors, 3, Callback, true, 0.25))
		group:AddChild(UIF:ColorSelect("Death", db.RuneColors, 4, Callback, true, 0.25))		
		group:AddChild(UIF:ColorSelect("Background Color", db, "FrameColor", Callback, true, 0.5))
		group:AddChild(UIF:LSMDropdown("statusbar", "Texture", db, "RuneTexture", Callback, 0.5))		
		
		group = UIF:InlineGroup1("Alpha")
		returnGroup:AddChild(group)
		group:AddChild(UIF:Slider("In combat", db, "RuneAlphaInCombat", 0, 1, 0.01, Callback, 0.33))
		group:AddChild(UIF:Slider("No combat (Runes on cooldown)", db, "RuneAlphaActive", 0, 1, 0.01, Callback, 0.33))
		group:AddChild(UIF:Slider("No combat (No runes on cooldown)", db, "RuneAlphaOutCombat", 0, 1, 0.01, Callback, 0.33))
		
		group = UIF:InlineGroup1("Font")
		returnGroup:AddChild(group)
		group:AddChild(UIF:Text1("These settings are for the cooldown text on each rune"))
		group:AddChild(UIF:LSMDropdown("font", "Font", db, "Font", Callback, 0.5))
		group:AddChild(UIF:Dropdown("Font Flags", db, "FontFlags", Callback, FontFlagList, 0.5))		
		group:AddChild(UIF:ColorSelect("Color", db, "FontColor", Callback, false, 0.2))
		group:AddChild(UIF:Slider("Size", db, "FontSize", 2, 50, 1, Callback, 0.4))
		group:AddChild(UIF:Slider("Spacing", db, "RuneCooldownInset", 0, 50, 0.1, Callback, 0.4))
		group:AddChild(UIF:Dropdown("Horizontal Align", db, "RuneCooldownHorAlign", Callback, HorizontalAlignList, 0.5))
		group:AddChild(UIF:Dropdown("Vertical Align", db, "RuneCooldownVerAlign", Callback, VerticalAlignList, 0.5))
		group:AddChild(UIF:CheckBox("Show", db, "RuneCooldownShow", Callback, "Toggle if the cooldown text should used and shown", 0.5))
		group:AddChild(UIF:CheckBox("Always Show", db, "RuneCooldownAlwaysShow", Callback, "Toggle if the cooldown text should always be shown", 0.5))
		
		return returnGroup
	end
end