local _, class = UnitClass("player")
if class ~= "DEATHKNIGHT" then
	return
end

local plugin = mRunes:NewModule("Runic Power", "AceEvent-3.0")
plugin.mRunesPlugin = true
MRUNES_RUNIC_POWER = plugin

local AceGUI = LibStub("AceGUI-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local UIF
local db

local InCombat = false
local HaveRP = false

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			x = 0,
			y = -21,
			Width = 337,
			Height = 20,
			ExternalSpacing = 1,
			InternalSpacing = 1,
			BackgroundColor = {0, 0, 0, 0.5},			
			AlphaIncCombat = 1,
			AlphaOutCombat = 0.2,
			AlphaActive = 0.5,
			
			HorAlign = "RIGHT",
			VerAlign = "BOTTOM",
			AlwaysShowText = false,
			ShowText = true,
			
			Font = "accid",
			FontSize = 16,
			FontFlags = "OUTLINE",
			FontColor = {1, 1, 1, 1},
			
			Texture = "HalD",
			PowerColor = {0, 0.82, 1, 1},
		}
	}
	
	self.db = mRunes.db:RegisterNamespace(self:GetName(), defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	db = self.db.profile
	
	self:SetEnabledState(db.Enabled)
	
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mRunes")
end

function plugin:OnEnable()	
	if not self.PowerFrame then
		self:CreatePowerFrame()
	end	
	
	self:ApplySettings()
	
	self:RegisterEvent("UNIT_MAXRUNIC_POWER", "RunicPowerChange")
	self:RegisterEvent("UNIT_RUNIC_POWER", "RunicPowerChange")
	mRunes.RegisterCallback(self, "EnteringCombat")
	mRunes.RegisterCallback(self, "LeavingCombat")
	
	if UnitAffectingCombat("player") then
		self:EnteringCombat("EnteringCombat")
	else
		self:LeavingCombat("LeavingCombat")
	end
	
	mRunes:RegisterOnUpdate(self.PowerFrame, self.RunicPowerChange, 1)
end

function plugin:OnDisable()
	mRunes:UnregisterOnUpdate(self.PowerFrame, self.RunicPowerChange)
end

function plugin:ProfileChanged()
	db = self.db.profile
	self:ApplySettings()
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Callbacks ~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:EnteringCombat(event)
	mRunes.FadeIn(self.PowerFrame, false, db.AlphaIncCombat)
	InCombat = true
	HaveRP = false
end

function plugin:LeavingCombat(event)
	if UnitPower("player") <= 0 then mRunes.FadeOut(self.PowerFrame, db.AlphaOutCombat) end
	InCombat = false
end

function plugin:EnterWorld(...)
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Events ~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:RunicPowerChange(event, unit)	
	self = plugin
	if unit and unit ~= "player" then return end
	local power = UnitPower("player")
	local maxPower = UnitPowerMax("player")
	local p = power / maxPower	
	local haveRP = false
	local oldHaveRP = HaveRP
	if power > 0 then
		haveRP = true
		self.PowerFrame.bar:SetValue(p)		
		self.PowerFrame.text:SetText(power.." / "..maxPower)
		if db.ShowText and not self.PowerFrame.text:IsShown() then self.PowerFrame.text:Show() end
	else
		self.PowerFrame.bar:SetValue(0)
		self.PowerFrame.text:SetText("0 / "..maxPower)
		if not db.AlwaysShowText then 
			if self.PowerFrame.text:IsShown() then self.PowerFrame.text:Hide() end
		elseif db.ShowText then
			if not self.PowerFrame.text:IsShown() then self.PowerFrame.text:Show() end
		else
			if self.PowerFrame.text:IsShown() then self.PowerFrame.text:Hide() end
		end		
	end
	
	if not InCombat then
		HaveRP = haveRP
		if HaveRP and (HaveRP ~= oldHaveRP) then
			local currAlpha = self.PowerFrame:GetAlpha()
			if currAlpha < db.AlphaActive then
				mRunes.FadeIn(self.PowerFrame, false, db.AlphaActive)				
			else
				mRunes.FadeOut(self.PowerFrame, db.AlphaActive)
				local t = 2
			end
		elseif not HaveRP and (HaveRP ~= oldHaveRP) then
			mRunes.FadeOut(self.PowerFrame, db.AlphaOutCombat)
		end
	end
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Settings	~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:ApplySettings()
	local PowerFrame = self.PowerFrame
	if PowerFrame then
		PowerFrame:ClearAllPoints()
		PowerFrame:SetPoint("CENTER", mRunes.MainFrame, "CENTER", db.x, db.y)
		PowerFrame:SetWidth(db.Width)
		PowerFrame:SetHeight(db.Height)
		
		PowerFrame.text:SetPoint("TOPLEFT", db.InternalSpacing, -db.InternalSpacing)
		PowerFrame.text:SetPoint("BOTTOMRIGHT", -db.InternalSpacing, db.InternalSpacing)
		PowerFrame.text:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontFlags)
		PowerFrame.text:SetTextColor(unpack(db.FontColor))
		PowerFrame.text:SetJustifyH(db.HorAlign)
		PowerFrame.text:SetJustifyV(db.VerAlign)
		
		local bar = PowerFrame.bar
		bar:ClearAllPoints()
		bar:SetPoint("TOPLEFT", db.ExternalSpacing, -db.ExternalSpacing)
		bar:SetPoint("BOTTOMRIGHT", -db.ExternalSpacing, db.ExternalSpacing)
		
		bar:SetStatusBarTexture(LSM:Fetch("statusbar", db.Texture))
		bar:GetStatusBarTexture():SetHorizTile(false)
		bar:SetStatusBarColor(unpack(db.PowerColor))
		
		PowerFrame.bg:SetTexture(unpack(db.BackgroundColor))
		
		PowerFrame:SetAlpha(InCombat and db.AlphaIncCombat or (HaveRP and db.AlphaActive or db.AlphaOutCombat))
		if PowerFrame:GetAlpha() > 0 then PowerFrame:Show() else PowerFrame:Hide() end		
		if db.AlwaysShowText and db.ShowText then
			PowerFrame.text:Show()
		else
			PowerFrame.text:Hide()				
		end
		
		self:RunicPowerChange("UNIT_RUNIC_POWER", "player")
	end	
	
	if not db.Enabled and self:IsEnabled() then self:Disable()
	elseif db.Enabled and not self:IsEnabled() then self:Enable() end
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Create Frames ~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:CreatePowerFrame()
	local frame = CreateFrame("Frame", nil, UIParent)
	local bg = frame:CreateTexture()
	bg:SetAllPoints()
	frame.bg = bg
	
	local bar = CreateFrame("StatusBar", nil, frame)
	bar:SetMinMaxValues(0,1)
	frame.bar = bar
	
	local text = bar:CreateFontString(nil, nil, "GameFontNormal")
	frame.text = text	
	
	self.PowerFrame = frame
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Options ~~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function plugin:GetOptions()
	return plugin.GetGUI
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	local FontFlagList = {
		["NONE"] = "None",
		["OUTLINE"] = "Outline",
		["THICKOUTLINE"] = "Thick Outline",
		["MONOCHROME"] = "Monochrome",
	}
	
	local VerticalAlignList = {
		["TOP"] = "Top",
		["MIDDLE"] = "Middle",
		["BOTTOM"] = "Bottom",
	}
	
	local HorizontalAlignList = {
		["LEFT"] = "Left",
		["CENTER"] = "Center",
		["RIGHT"] = "Right",
	}

	
	function plugin:GetGUI()
		self = plugin
		
		local returnGroup = AceGUI:Create("SimpleGroup")
		returnGroup:SetFullWidth(true)
		returnGroup:SetLayout("Flow")
		
		returnGroup:AddChild(UIF:CheckBox("Enabled", db, "Enabled", Callback, nil, 0.33))
		
		local group = UIF:InlineGroup1("Position and Size")
		returnGroup:AddChild(group)		
		group:AddChild(UIF:Text1("Positions here are relative to the mRunes main position"))
		group:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.5))
		group:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.5))
		group:AddChild(UIF:Slider("Width", db, "Width", 5, 1000, 0.1, Callback, 0.5))
		group:AddChild(UIF:Slider("Height", db, "Height", 5, 200, 0.1, Callback, 0.5))
		group:AddChild(UIF:Slider("Background Space", db, "ExternalSpacing", 0, 100, 0.1, Callback, 0.5))
		
		group = UIF:InlineGroup1("Colors")
		returnGroup:AddChild(group)
		group:AddChild(UIF:ColorSelect("Runic Power color", db, "PowerColor", Callback, true, 0.25))		
		group:AddChild(UIF:ColorSelect("Background Color", db, "BackgroundColor", Callback, true, 0.25))
		group:AddChild(UIF:LSMDropdown("statusbar", "Texture", db, "Texture", Callback, 0.5))		
		
		group = UIF:InlineGroup1("Alpha")
		returnGroup:AddChild(group)
		group:AddChild(UIF:Slider("In combat", db, "AlphaIncCombat", 0, 1, 0.01, Callback, 0.33))
		group:AddChild(UIF:Slider("No combat (RP > 0)", db, "AlphaActive", 0, 1, 0.01, Callback, 0.33))
		group:AddChild(UIF:Slider("No Combat (RP = 0)", db, "AlphaOutCombat", 0, 1, 0.01, Callback, 0.33))
		
		group = UIF:InlineGroup1("Font")
		returnGroup:AddChild(group)
		group:AddChild(UIF:Text1("These settings are for the cooldown text on each rune"))
		group:AddChild(UIF:LSMDropdown("font", "Font", db, "Font", Callback, 0.5))
		group:AddChild(UIF:Dropdown("Font Flags", db, "FontFlags", Callback, FontFlagList, 0.5))		
		group:AddChild(UIF:ColorSelect("Color", db, "FontColor", Callback, false, 0.2))
		group:AddChild(UIF:Slider("Size", db, "FontSize", 2, 50, 1, Callback, 0.4))
		group:AddChild(UIF:Slider("Spacing", db, "InternalSpacing", 0, 50, 0.1, Callback, 0.4))
		group:AddChild(UIF:Dropdown("Horizontal Align", db, "HorAlign", Callback, HorizontalAlignList, 0.5))
		group:AddChild(UIF:Dropdown("Vertical Align", db, "VerAlign", Callback, VerticalAlignList, 0.5))
		group:AddChild(UIF:CheckBox("Show", db, "ShowText", Callback, "Toggle if the runic power text should be shown.", 0.5))
		group:AddChild(UIF:CheckBox("Always Show", db, "AlwaysShowText", Callback, "Toggle if the runic power text should always be shown. If not it will be hidden when the runc power is at 0", 0.5))
		
		return returnGroup
	end
end